/****************************************************************
 * See file COPYING included in distribution for license details
 ****************************************************************/
#ifndef _BOX_MAP_H_
#define _BOX_MAP_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "coordmap.h"
#include "graph_types.h"
#include "graph_properties.h"
#include "db-interfaces/interface_defs.h"

typedef enum {TEXT_JUSTIFY_FONTSTYLE, TEXT_JUSTIFY_ESTIMATE_1 } TEXT_JUSTIFY_MODE;

typedef struct DBFieldLocation_struct {
   char  *value;
   float position;
} DBFieldLocation;

typedef struct BoxGraphAppearance_struct {
   SFColor           box_color;
   SimpleGraphItem   graph_lines;
   TEXT_JUSTIFY_MODE text_justify_mode;
   SimpleGraphItem   x_axis_labels;
   SimpleGraphItem   x_axis_heading;
   SimpleGraphItem   y_axis_values;
   SimpleGraphItem   z_axis_labels;
   SimpleGraphItem   z_axis_heading;
   SimpleGraphItem   elevation_grid;
} BoxGraphAppearance;

typedef struct DBFieldOrder_struct {
   char                  *field_name;
   int                   field_order;
   DBFieldLocation       **axis;
   int                   n_axis;
} DBFieldOrder;

typedef struct BoxMap_struct {
   CoordMap *x_axis;
   CoordMap *y_axis;
   CoordMap *z_axis;
} BoxMap;


typedef struct BoxGraphData_struct {
   GraphType          type;
   BoxGraphAppearance appearance;
   BoxMap             *box_map;
   DBFieldOrder       **fields;
} BoxGraphData;

// DBFieldLocation constructors/destructors
DBFieldLocation* DBFieldLocation_new    (const char* value, float position );
void             DBFieldLocation_delete (DBFieldLocation *p);
// BoxMap constructors/destructors
BoxMap*          BoxMap_new                     ();
void             BoxMap_delete                  (BoxMap*);
// Shallow replace of CoordMap structs
void             BoxMap_replace_CoordMap_N      (BoxMap*, CoordMap*, int x_y_z );
#define          BoxMap_replace_CoordMap_X(b,m)  BoxMap_replace_CoordMap_N(b,m,0)
#define          BoxMap_replace_CoordMap_Y(b,m)  BoxMap_replace_CoordMap_N(b,m,1)
#define          BoxMap_replace_CoordMap_Z(b,m)  BoxMap_replace_CoordMap_N(b,m,2)

BoxGraphData*    BoxGraphData_new            ();
void             BoxGraphData_delete         (BoxGraphData*);
// Shallow replace
void             BoxGraphData_replace_BoxMap (BoxGraphData*, BoxMap*);

int              graph_box                   (BoxGraphData* data, DATA_ACCESS_FNXS *fnxs, const char* file_name );

#ifdef __cplusplus
}
#endif
   
#endif
