#include "interface_defs.h"
#include <string.h>
#include <stdlib.h>
#include <assert.h>


/***************  DATA_ACCESS_FNXS_set_functions  ******************/
void
DATA_ACCESS_FNXS_set_functions  (DATA_ACCESS_FNXS   *d,
                                 DB_INITIALIZE      init,
                                 DB_CLOSE           close,
                                 DB_NUM_FIELDS_FNX  n_fields,
                                 DB_NUM_TUPLES_FNX  n_tuples,
                                 DB_FIELD_NAME_FNX  field_name,
                                 DB_GET_VALUE_FNX   get_value,
                                 DB_IS_NULL_FNX     is_null
                                 )
{
   d->initialize_fnx    = init;
   d->close_fnx         = close;
   d->num_fields_fnx    = n_fields;
   d->num_tuples_fnx    = n_tuples;
   d->field_name_fnx    = field_name;
   d->get_value_fnx     = get_value;
   d->is_null_fnx       = is_null;
}


/****************  DB_Init_Data_new  ***********************/
DB_Init_Data*
DB_Init_Data_new ( const char* data_base,
                   const char* relation,
                   const char* host_name,
                   const char* port,
                   const char* server_options)
{
   DB_Init_Data *d = (DB_Init_Data*)malloc( sizeof(DB_Init_Data) );
   
   d->data_base      = data_base      ? strdup(data_base)      : NULL;
   d->relation       = relation       ? strdup(relation)       : NULL;
   d->host_name      = host_name      ? strdup(host_name)      : NULL;
   d->port           = port           ? strdup(port)           : NULL;
   d->server_options = server_options ? strdup(server_options) : NULL;
 
   return d;
}

/***************  DB_Init_Data_free_members  ****************/
void
DB_Init_Data_free_members ( DB_Init_Data *d )
{
   if ( d ) {
      if ( d->data_base )
         free (d->data_base);
      if ( d->relation )
         free (d->relation);
      if ( d->host_name )
         free (d->host_name);
      if ( d->port )
         free (d->port);
      if ( d->server_options )
         free (d->server_options);
   }
}

/***************  DB_Init_Data_delete  **********************/
void
DB_Init_Data_delete ( DB_Init_Data *d )
{
   if ( d ) {
      DB_Init_Data_free_members(d);
      free (d);
   }
   
}

/*****************  DB_Init_Data_copy ***********************/
void
DB_Init_Data_copy ( DB_Init_Data *to, DB_Init_Data *from)
{
   if ( to && from ) {
      DB_Init_Data_free_members(to);
      
      to->data_base      = from->data_base      ? strdup(from->data_base)      : NULL;
      to->relation       = from->relation       ? strdup(from->relation)       : NULL;
      to->host_name      = from->host_name      ? strdup(from->host_name)      : NULL;
      to->port           = from->port           ? strdup(from->port)           : NULL;
      to->server_options = from->server_options ? strdup(from->server_options) : NULL;
   }
   
}

/***************  DB_Instance_Data_new  ********************/
DB_Instance_Data*
DB_Instance_Data_new()
{
   DB_Instance_Data  *d = (DB_Instance_Data*)malloc(sizeof(DB_Instance_Data));

   d->connection_information = NULL;

   d->init_data = DB_Init_Data_new(NULL, NULL, NULL, NULL, NULL );
   
   return d;
}

/************  DB_Instance_Data_delete  ***************/
void
DB_Instance_Data_delete ( DB_Instance_Data *d)
{
   // This is specific to the DB interface and allocation
   // de-allocation should be handled there.

   DB_Init_Data_delete (d->init_data);
}

/**************  DB_Instance_Data_use_shallow_ptr  ******************/
void
DB_Instance_Data_use_shallow_ptr ( DB_Instance_Data *inst, DB_Init_Data *init)
{
   assert (inst);
   assert (init);

   DB_Init_Data_delete( inst->init_data );
   inst->init_data = init;
   
}
