#ifndef _INTERFACE_DEFS_H
#define _INTERFACE_DEFS_H

#ifdef __cplusplus
extern "C" {
#endif

#define VRMLDB_MAJOR_VERSION_NUMBER 0
#define VRMLDB_MINOR_VERSION_NUMBER 1

   
typedef struct DB_Init_Data_struct {
   char  *data_base;
   char  *relation;
   char  *host_name;
   char  *port;
   char  *server_options;
} DB_Init_Data;

   
// May not need all of these variables but the interface
// initializer / destructor can do with them what they will
typedef struct DB_Instance_Data_struct {
   int              major_version_number;
   int              minor_version_number;
   void             *connection_information;
   DB_Init_Data     *init_data;
} DB_Instance_Data;


// Initialize and store any pertinent data in DB_Instance_Data struct
typedef int   (*DB_INITIALIZE)       ( DB_Instance_Data * );
   
// Close Database connection and free memory
typedef void  (* DB_CLOSE          ) ( DB_Instance_Data * );
                                      
typedef int   (* DB_NUM_FIELDS_FNX ) ( DB_Instance_Data *);
typedef int   (* DB_NUM_TUPLES_FNX ) ( DB_Instance_Data *);
typedef char* (* DB_FIELD_NAME_FNX ) ( DB_Instance_Data *, int field_number);
typedef char* (* DB_GET_VALUE_FNX  ) ( DB_Instance_Data *, int tuple,  int field_order );
typedef int   (* DB_IS_NULL_FNX    ) ( DB_Instance_Data *, int tuple,  int field_order );

   
typedef struct DATA_ACCESS_FNXS_struct {
   DB_Instance_Data  *instance_data;
   DB_INITIALIZE     initialize_fnx;
   DB_CLOSE          close_fnx;
   DB_NUM_FIELDS_FNX num_fields_fnx;
   DB_NUM_TUPLES_FNX num_tuples_fnx;
   DB_FIELD_NAME_FNX field_name_fnx;
   DB_GET_VALUE_FNX  get_value_fnx;
   DB_IS_NULL_FNX    is_null_fnx;
} DATA_ACCESS_FNXS;

// Set the access functions
void DATA_ACCESS_FNXS_set_functions                (DATA_ACCESS_FNXS *,
                                                    DB_INITIALIZE,
                                                    DB_CLOSE,
                                                    DB_NUM_FIELDS_FNX,
                                                    DB_NUM_TUPLES_FNX,
                                                    DB_FIELD_NAME_FNX,
                                                    DB_GET_VALUE_FNX,
                                                    DB_IS_NULL_FNX);
// DB_Instance_Data functions
DB_Instance_Data* DB_Instance_Data_new             ();
void              DB_Instance_Data_delete          ( DB_Instance_Data * );
void              DB_Instance_Data_use_shallow_ptr ( DB_Instance_Data *, DB_Init_Data * );
// DB_Init_Data functions
DB_Init_Data*     DB_Init_Data_new                 ( const char* data_base,
                                                     const char* relation,
                                                     const char* host_name,
                                                     const char* port,
                                                     const char* server_options);
void              DB_Init_Data_delete              ( DB_Init_Data* );
void              DB_Init_Data_copy                ( DB_Init_Data *to, DB_Init_Data *from);
void              DB_Init_Data_free_members        ( DB_Init_Data* );
   
   
#ifdef __cplusplus
}
#endif

#endif

